;; Generated package description from ebdb-i18n-chn.el  -*- no-byte-compile: t -*-
(define-package "ebdb-i18n-chn" "1.3.2" "China-specific internationalization support for EBDB" '((pyim "1.6.0") (ebdb "0.6.17")) :commit "b8c1f7c21204bda9c130ce21a75d69358d1bc49a" :url "https://elpa.gnu.org/packages/ebdb-i18n-chn.html" :authors '(("Eric Abrahamsen" . "eric@ericabrahamsen.net")) :maintainer '("Eric Abrahamsen" . "eric@ericabrahamsen.net"))
